==> geturl-1.0.tar.gz: geturl-1.0/connect.c <==
/* connect.c
   Module for handling of generic network connections */

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/nameser.h>
#include <netinet/in.h>
#include <errno.h>
#ifdef HAVE_STRING_H
#  include <string.h>
#else
#  include <strings.h>
#endif /* HAVE_STRING_H */

#include "geturl.h"
#include "connect.h"

/* Global variables for sharing between bindport
   and acceptport: */
int msock, addrlen;
struct sockaddr *addr;
struct sockaddr_in srv;


/* Thanks to Miroslav Zagorac <zaga@fly.cc.fer.hr> */
uerr_t
make_connection(int *sock, char *hostname, int port)
{
   struct sockaddr_in sock_name;
   struct hostent *hptr;

   /* Get internet address of the host */
   if (!(hptr = gethostbyname(hostname)))
      return HOSTERR;

   /* Set port and protocol */
   sock_name.sin_family = AF_INET;
   sock_name.sin_port = htons(port);
   memcpy(&sock_name.sin_addr, hptr->h_addr, hptr->h_length);
   /* Make internet socket */
   if ((*sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
      return CONSOCKERR;
   
   /* Connect to remote host */
   if (connect(*sock, (struct sockaddr *) &sock_name, sizeof (sock_name)))
   {
      if (errno == ETIMEDOUT)
	 return CONTIMEDOUT;
      else if (errno == ECONNREFUSED)
	 return CONREFUSED;
      else
	 return CONERROR;
   }
   return NOCONERROR;
}

/* Thanks to Zlatko Calusic <maverick@fly.cc.fer.hr> */

/* bindport does all the work necessary to bind a port, creating a
   master socket msock. Use acceptport to call accept and receive the
   actual socket. */
uerr_t
bindport(int *port)
{
   int optval = 1;

   addr = (struct sockaddr *) & srv;
   addrlen = sizeof(srv);
   if ((msock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
      return CONSOCKERR;
   if (setsockopt(msock, SOL_SOCKET, SO_REUSEADDR, (char *)&optval, sizeof(optval)) < 0)
      return CONSOCKERR;
   srv.sin_family = AF_INET;
   srv.sin_addr.s_addr = htonl(INADDR_ANY);
   srv.sin_port = htons(*port);
   if (bind(msock, addr, addrlen) < 0)
      return BINDERR;
   if (!*port)
   {
      if (getsockname(msock, addr, &addrlen) < 0)
	 return CONPORTERR;
      *port = ntohs(srv.sin_port);
   }
   if (listen(msock, 1) < 0)
      return LISTENERR;
   return BINDOK;
}

/* Acceptport uses msock and addrlen to call the accept system call
   and return the data socket of the connection. It blocks the caller,
   as per accept. */
uerr_t
acceptport(int *sock)
{
   if ((*sock = accept(msock, addr, &addrlen)) < 0)
      return ACCEPTERR;
   return ACCEPTOK;
}

/* Calls shutdown and closes the sockets. */
void
closeport(int sock)
{
   shutdown(sock, 2);
   close(sock);
   close(msock);
}

==> geturl-1.1.tar.gz: geturl-1.1/connect.c <==
/* connect.c
   Module for handling of generic network connections */

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/nameser.h>
#include <netinet/in.h>
#include <errno.h>
#if TIME_WITH_SYS_TIME
#  include <sys/time.h>
#  include <time.h>
#else
#if HAVE_SYS_TIME_H
#  include <sys/time.h>
#else
#  include <time.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#else
#  include <strings.h>
#endif /* HAVE_STRING_H */
#endif

#include "geturl.h"
#include "connect.h"
#include "host.h"

#ifdef HAVE_SELECT
extern int timeout;
#endif

/* Global variables for sharing between bindport
   and acceptport: */
int msock, addrlen;
struct sockaddr *addr;
struct sockaddr_in srv;


/* Thanks to Miroslav Zagorac <zaga@fly.cc.fer.hr> */
uerr_t
make_connection(int *sock, char *hostname, int port)
{
   struct sockaddr_in sock_name;
   struct hostent *hptr;

   /* Get internet address of the host */
   if (!(hptr = ngethostbyname(hostname)))
      return HOSTERR;

   /* Set port and protocol */
   sock_name.sin_family = AF_INET;
   sock_name.sin_port = htons(port);
   memcpy(&sock_name.sin_addr, hptr->h_addr, hptr->h_length);
   /* Make internet socket */
   if ((*sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
      return CONSOCKERR;
   
   /* Connect to remote host */
   if (connect(*sock, (struct sockaddr *) &sock_name, sizeof (sock_name)))
   {
      if (errno == ETIMEDOUT)
	 return CONTIMEDOUT;
      else if (errno == ECONNREFUSED)
	 return CONREFUSED;
      else
	 return CONERROR;
   }
   return NOCONERROR;
}

/* Thanks to Zlatko Calusic <maverick@fly.cc.fer.hr> */

/* bindport does all the work necessary to bind a port, creating a
   master socket msock. Use acceptport to call accept and receive the
   actual socket. */
uerr_t
bindport(int *port)
{
   int optval = 1;

   addr = (struct sockaddr *) & srv;
   addrlen = sizeof(srv);
   if ((msock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
      return CONSOCKERR;
   if (setsockopt(msock, SOL_SOCKET, SO_REUSEADDR, (char *)&optval, sizeof(optval)) < 0)
      return CONSOCKERR;
   srv.sin_family = AF_INET;
   srv.sin_addr.s_addr = htonl(INADDR_ANY);
   srv.sin_port = htons(*port);
   if (bind(msock, addr, addrlen) < 0)
      return BINDERR;
   if (!*port)
   {
      if (getsockname(msock, addr, &addrlen) < 0)
	 return CONPORTERR;
      *port = ntohs(srv.sin_port);
   }
   if (listen(msock, 1) < 0)
      return LISTENERR;
   return BINDOK;
}

/* Acceptport uses msock and addrlen to call the accept system call
   and return the data socket of the connection. It blocks the caller,
   as per accept. */
uerr_t
acceptport(int *sock)
{
   if ((*sock = accept(msock, addr, &addrlen)) < 0)
      return ACCEPTERR;
   return ACCEPTOK;
}

/* Calls shutdown and closes the sockets. */
void
closeport(int sock)
{
   shutdown(sock, 2);
   close(sock);
   close(msock);
}

#ifdef HAVE_SELECT
/* select_fd - Waits for file descriptor fd to be readable, where
 maxtime is the timeout in seconds. Returns 1 if fd is readable, 0 for
 timeout and -1 for error. */
int select_fd(int fd, int maxtime)
{
   fd_set readfds;
   fd_set exceptfds;
   struct timeval timeout;
   int nfds;
   
   FD_ZERO(&readfds);
   FD_SET(fd, &readfds);
   FD_ZERO(&exceptfds);
   FD_SET(fd, &exceptfds);
   timeout.tv_sec = maxtime;
   timeout.tv_usec = 0;
   nfds = select(255, &readfds, NULL, &exceptfds, &timeout);
   return nfds;
}
#endif /* HAVE_SELECT */

/* The same like read, but takes care of EINTR and uses select if
   necessary. */
int iread(int fd, char *buf, int len)
{
   int res;
   
   while (1)
   {
#ifdef HAVE_SELECT
      if (timeout)
      {
         res = select_fd(fd, timeout);
         if (res != 1)
            return -1;
      }
#endif
      res = read(fd, buf, len);
      if (res == -1 && errno == EINTR)
         continue;
      else
         break;
   }
   return res;
}
